#pragma once

#include "messagebasic_global.h"
#include <simsbasic.h>
#include <standardglobe.h>

#define NAME(v) #v
#define DOUBLE_SAVE(u) obj[NAME(u)] = u
#define DOUOBLE_PEAK(u) u = obj[NAME(u)].toDouble()

#define STRING_SAVE(u) obj[NAME(u)] = u
#define STRING_PEAK(u) u = obj[NAME(u)].toString()

#define STRLIST_SAVE(u) obj[NAME(u)] = u.join(",")
#define STRLIST_PEAK(u) u = obj[NAME(u)].toString().split(",")

class MESSAGEBASIC_EXPORT AbstractMessage : public WsMessage {
private:
	uint64_t _from_id = 0, _to_id = 0;
	QString _topic_string;

public:
	AbstractMessage(const QString& topic);
	virtual ~AbstractMessage() = default;

	void reset(uint64_t from, uint64_t to);

	// ͨ WsMessage ̳
	void recoveryFrom(const QJsonObject& obj) override;
	void saveTo(QJsonObject& obj) const override;
	QString topicString() const override;
	uint64_t targetEntity() const override;
	uint64_t sourceEntity() const override;
};

/// <summary>
/// 
/// </summary>
struct MESSAGEBASIC_EXPORT DeduceRequest : public AbstractMessage {
	DeduceRequest();
};

/// <summary>
/// Ĭϵ޷ϢĴͨ÷Ϣ
/// </summary>
struct MESSAGEBASIC_EXPORT RespondDefault : public AbstractMessage {
	double _time_consume;

	RespondDefault();

	// ͨ WsMessage ̳
	void recoveryFrom(const QJsonObject& obj) override;
	void saveTo(QJsonObject& obj) const override;
};

/// <summary>
/// ʱͬϢ
/// </summary>
struct MESSAGEBASIC_EXPORT SyncRequest : public AbstractMessage {
	double _time_current;

	SyncRequest();

	// ͨ WsMessage ̳
	void recoveryFrom(const QJsonObject& obj) override;
	void saveTo(QJsonObject& obj) const override;
};

// пӻϢ =======================================================
/// <summary>
/// ȡϸ
/// </summary>
struct MESSAGEBASIC_EXPORT Get3DBox : public AbstractMessage {
	Get3DBox();
};

/// <summary>
/// ̬ݽṹ
/// </summary>
struct Posture {
	double _azimuth_deg = 0;
	double _pitch_deg = 0;
	double _roll_deg = 0;
};
/// <summary>
/// άݽṹ
/// </summary>
struct D3Data {
	double _length_m = 0;
	double _width_m = 0;
	double _height_m = 0;
};

/// <summary>
/// ϸݶ
/// </summary>
struct MESSAGEBASIC_EXPORT Box3DDesc : public AbstractMessage {
	D3Data _d3_data;
	Posture _posture_d3;
	LLAPos _lla_pos;

	Box3DDesc();

	// ͨ WsMessage ̳
	void recoveryFrom(const QJsonObject& obj) override;
	void saveTo(QJsonObject& obj) const override;
};

// ʵ幹Ϣ ==========================================

/// <summary>
/// ʵ幹
/// </summary>
struct MESSAGEBASIC_EXPORT EntityAllocte : public AbstractMessage {
	QString _entity_templet = "";

	EntityAllocte();

	// ͨ WsMessage ̳
	void recoveryFrom(const QJsonObject& obj) override;
	void saveTo(QJsonObject& obj) const override;
};

/// <summary>
/// ȡʵ
/// </summary>
struct MESSAGEBASIC_EXPORT EntityTotalGet : public AbstractMessage {
	EntityTotalGet();
};
/// <summary>
/// ʵб
/// </summary>
struct MESSAGEBASIC_EXPORT EntityTotalList : public AbstractMessage {
	QList<uint64_t> _entities_list;

	EntityTotalList();

	// ͨ WsMessage ̳
	void recoveryFrom(const QJsonObject& obj) override;
	void saveTo(QJsonObject& obj) const override;
};